@@ -18,7 +18,7 @@ along with pdsnd; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
-/* $Id: conff.h,v 1.18 2001/05/09 17:51:52 tmm Exp $ */
+/* $Id: conff.h,v 1.19 2001/07/01 21:03:15 tmm Exp $ */
 
 #ifndef CONFF_H
 #define CONFF_H
@@ -92,6 +92,8 @@ typedef struct {
 	int           tcp_qtimeout;
 	int           par_queries;
 	char          rnd_recs;
+	int           query_port_start;
+	int           query_port_end;
 } globparm_t;
 
 extern globparm_t global;
