@@ -23,11 +23,12 @@ Boston, MA 02111-1307, USA.  */
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include "error.h"
 #include "helpers.h"
 #include "dns.h"
 
 #if !defined(lint) && !defined(NO_RCSIDS)
-static char rcsid[]="$Id: dns.c,v 1.28 2001/07/02 18:53:37 tmm Exp $";
+static char rcsid[]="$Id: dns.c,v 1.31 2002/01/03 17:47:20 tmm Exp $";
 #endif
 
 /* Decompress a name record, taking the whole message as msg, returning its results in tgt (max. 255 chars),
@@ -175,21 +176,21 @@ int domain_match(int *o, unsigned char *ms, unsigned char *md, unsigned char *re
  * offs is the offset the generated string will be placed in the packet.
  * retval: 0 - error, otherwise length
  * When done, just free() cb (if it is NULL, free will behave correctly).
+ * It is guaranteed (and insured by assertions) that the output is smaller or equal in
+ * size to the input.
  */
 int compress_name(unsigned char *in, unsigned char *out, int offs, darray *cb)
 {
-#if 0
-	/* Delete this debug code when done */
-	unsigned char buf1[256],buf2[256];
-#endif
 	int i;
 	int add=1;
 	int coffs=-1;
 	int rv,rl,to;
 	int longest=0;
+	long ilen;
 	unsigned char rest[256];
 	unsigned char brest[256];
 	rl=0;
+	ilen = rhnlen(in);
 	/* part 1: compression */
 	if (*cb) {
 		for (i=0;i<da_nel(*cb);i++) {
@@ -207,30 +208,17 @@ int compress_name(unsigned char *in, unsigned char *out, int offs, darray *cb)
 			} 
 		}
 		if (coffs>-1) {
+			PDNSD_ASSERT(rhnlen(brest) + 1 <= ilen, "compress_name: length increased");
 			rl=rhncpy(out, brest)-1; /* omit the length byte, because it needs to be frobbed */
+			PDNSD_ASSERT(rl <= 254, "compress_name: name too long");
 			out[rl]=192|((coffs&0x3f00)>>8);
 			out[rl+1]=coffs&0xff;
 			rl+=2;
-#if 0
-			rhn2str(in,buf1);
-			rhn2str(brest,buf2);
-			printf("Compressed %s to %s and reference to %i.\n",buf1,buf2,coffs);
-#endif
 			add=strlen((char *)brest)!=0;
-		} else {
-#if 0
-			rhn2str(in,buf1);
-			printf("%s not compressed.\n",buf1);
-#endif
+		} else
 			rl=rhncpy(out,in);
-		}
-	} else {
-#if 0
-		rhn2str(in,buf1);
-		printf("%s not compressed.\n",buf1);
-#endif
+	} else
 		rl=rhncpy(out,in);
-	}
 
 	/* part 2: addition to the cache structure */
 	if (add) {
