@@ -56,7 +56,7 @@ Boston, MA 02111-1307, USA.  */
 #include "debug.h"
 
 #if !defined(lint) && !defined(NO_RCSIDS)
-static char rcsid[]="$Id: dns_answer.c,v 1.54 2001/06/03 21:11:43 tmm Exp $";
+static char rcsid[]="$Id: dns_answer.c,v 1.58 2002/01/04 14:53:06 tmm Exp $";
 #endif
 
 /*
@@ -242,6 +242,7 @@ static int add_rr(dns_hdr_t **ans, long *sz, rr_bucket_t *rr, unsigned short typ
 		}
 		*sz+=rrh.rdlength;
 		ilen=rhnlen((unsigned char *)(rr+1));
+		PDNSD_ASSERT(rrh.rdlength <= ilen, "T_MINFO/T_RP: got longer");
 		if (!(blen=compress_name(((unsigned char *)(rr+1))+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb))) {
 			pdnsd_free(*ans);
 			return 0;
@@ -255,6 +256,7 @@ static int add_rr(dns_hdr_t **ans, long *sz, rr_bucket_t *rr, unsigned short typ
 	case T_RT:
 	case T_KX:
 #endif
+		PDNSD_ASSERT(rr->rdlen > 2, "T_MX/T_AFSDB/...: rr botch");
 		memcpy(((unsigned char *)(*ans))+(*sz),(unsigned char *)(rr+1),2);
 		*sz+=2;
 		if (!(blen=compress_name(((unsigned char *)(rr+1))+2, ((unsigned char *)(*ans))+(*sz),*sz,cb))) {
@@ -271,19 +273,22 @@ static int add_rr(dns_hdr_t **ans, long *sz, rr_bucket_t *rr, unsigned short typ
 		}
 		*sz+=rrh.rdlength;
 		ilen=rhnlen((unsigned char *)(rr+1));
+		PDNSD_ASSERT(rrh.rdlength <= ilen, "T_SOA: got longer");
 		if (!(blen=compress_name(((unsigned char *)(rr+1))+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb))) {
 			pdnsd_free(*ans);
 			return 0;
 		}
 		rrh.rdlength+=blen;
 		*sz+=blen;
-		ilen+=strlen(((char *)(rr+1))+ilen)+1;
+		ilen+=rhnlen(((unsigned char *)(rr+1))+ilen);
+		PDNSD_ASSERT(rrh.rdlength <= ilen, "T_SOA: got longer");
 		memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)(rr+1))+ilen,sizeof(soa_r_t));
 		*sz+=sizeof(soa_r_t);
 		rrh.rdlength+=sizeof(soa_r_t);
 		break;
 #ifdef DNS_NEW_RRS
 	case T_PX:
+		PDNSD_ASSERT(rr->rdlen > 2, "T_PX: rr botch");
 		memcpy(((unsigned char *)(*ans))+(*sz),(unsigned char *)(rr+1),2);
 		*sz+=2;
 		ilen=2;
@@ -293,7 +298,8 @@ static int add_rr(dns_hdr_t **ans, long *sz, rr_bucket_t *rr, unsigned short typ
 		}
 		rrh.rdlength=2+blen;
 		*sz+=blen;
-		ilen+=rhnlen((unsigned char *)(rr+1));
+		ilen+=rhnlen(((unsigned char *)(rr+1))+ilen);
+		PDNSD_ASSERT(rrh.rdlength <= ilen, "T_PX: got longer");
 		if (!(blen=compress_name(((unsigned char *)(rr+1))+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb))) {
 			pdnsd_free(*ans);
 			return 0;
@@ -302,6 +308,7 @@ static int add_rr(dns_hdr_t **ans, long *sz, rr_bucket_t *rr, unsigned short typ
 		*sz+=blen;
 		break;
 	case T_SRV:
+		PDNSD_ASSERT(rr->rdlen > 6, "T_SRV: rr botch");
 		memcpy(((unsigned char *)(*ans))+(*sz),(unsigned char *)(rr+1),6);
 		*sz+=6;
 		if (!(blen=compress_name(((unsigned char *)(rr+1))+6, ((unsigned char *)(*ans))+(*sz),*sz,cb))) {
@@ -312,25 +319,29 @@ static int add_rr(dns_hdr_t **ans, long *sz, rr_bucket_t *rr, unsigned short typ
 		*sz+=blen;
 		break;
 	case T_NXT:
-		if (!(blen=compress_name(((unsigned char *)(rr+1))+6, ((unsigned char *)(*ans))+(*sz),*sz,cb))) {
+		if (!(blen=compress_name(((unsigned char *)(rr+1)), ((unsigned char *)(*ans))+(*sz),*sz,cb))) {
 			pdnsd_free(*ans);
 			return 0;
 		}
 		rrh.rdlength=blen;
 		*sz+=blen;
 		ilen=rhnlen((unsigned char *)(rr+1));
-		wlen=(rr->rdlen-ilen)<0?0:(rr->rdlen-ilen);
+		PDNSD_ASSERT(rrh.rdlength <= ilen, "T_NXT: got longer");
+		PDNSD_ASSERT(rr->rdlen >= ilen, "T_NXT: rr botch");
+		wlen=rr->rdlen < ilen ? 0 : (rr->rdlen - ilen);
 		memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)(rr+1))+ilen,wlen);
 		*sz+=wlen;
 		rrh.rdlength+=wlen;
 		break;
 	case T_NAPTR:
+		PDNSD_ASSERT(rr->rdlen > 5, "T_NAPTR: rr botch");
 		memcpy(((unsigned char *)(*ans))+(*sz),(unsigned char *)(rr+1),4);
 		*sz+=4;
 		ilen=4;
 		for (j=0;j<3;j++) {
 			k=*(((unsigned char *)(rr+1))+ilen);
-			memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)(rr+1))+ilen,k);
+			PDNSD_ASSERT(k + 1 + ilen < rr->rdlen, "T_NAPTR: rr botch 2");
+			memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)(rr+1))+ilen,k+1);
 			(*sz)+=k+1;
 			ilen+=k+1;
 		}
@@ -387,7 +398,8 @@ typedef struct rre_s {
 static int add_ar(void *tnm, int tsz, darray *ar, unsigned char *nm, time_t ts, time_t ttl, int flags, int tp)
 {
 	rr_ext_t *re;
-	
+
+	PDNSD_ASSERT(tsz <= 256, "add_ar: tsz botch");
 	if ((*ar=da_grow(*ar,1))==NULL) {
 		return 0;
 	}
