@@ -39,7 +39,7 @@ Boston, MA 02111-1307, USA.  */
 #include "conff.h"
 
 #if !defined(lint) && !defined(NO_RCSIDS)
-static char rcsid[]="$Id: helpers.c,v 1.29 2001/06/03 21:11:43 tmm Exp $";
+static char rcsid[]="$Id: helpers.c,v 1.33 2002/01/03 13:33:06 tmm Exp $";
 #endif
 
 /*
@@ -48,6 +48,7 @@ static char rcsid[]="$Id: helpers.c,v 1.29 2001/06/03 21:11:43 tmm Exp $";
 void pdnsd_exit()
 {
 	pthread_kill(main_thread,SIGTERM);
+	pthread_exit(NULL);
 }
 
 /*
@@ -121,11 +122,15 @@ int str2rhn(unsigned char *str, unsigned char *rhn)
 	
 	do {
 		lb=0;
+		if (lb + cnt >= 255)
+			return 0;
 		while(isdchar(str[lb+cnt])) {
 			if (lb>62)
 				return 0;
 			buf[lb]=str[lb+cnt];
 			lb++;
+			if (lb + cnt >= 255)
+				return 0;
 		}
 		if (str[lb+cnt]=='\0') {
 			if (lb==0) {
@@ -171,9 +176,13 @@ void rhn2str(unsigned char *rhn, unsigned char *str)
 	}
  	while (lb) {
 		for (i=0;i<lb;i++) {
+			PDNSD_ASSERT(cnt < 255,
+			    "rhn2str: string length overflow");
 			str[cnt-1]=rhn[cnt];
 			cnt++;
 		}
+		PDNSD_ASSERT(cnt <= 255,
+		    "rhn2str: string length overflow");
 		str[cnt-1]='.';
 		str[cnt]='\0';
 		lb=rhn[cnt];
@@ -215,6 +224,7 @@ int follow_cname_chain(dns_cent_t *c, unsigned char *name, unsigned char *rrn)
 	if (!(c->rr[T_CNAME-T_MIN] && c->rr[T_CNAME-T_MIN]->rrs))
 		return 0;
 	rr=c->rr[T_CNAME-T_MIN]->rrs;
+	PDNSD_ASSERT(rr->rdlen <= 256, "follow_cname_chain: record too long");
 	memcpy(rrn,rr+1,rr->rdlen);
 	rhn2str(rrn,name);
 	return 1;
@@ -300,7 +310,7 @@ char *socka2str(struct sockaddr *a, char *str, int maxlen)
 
 /* Appropriately set our random device */
 #ifdef R_DEFAULT
-# if TARGET == TARGET_BSD
+# if TARGET == TARGET_BSD && !defined(__NetBSD__)
 #  define R_ARC4RANDOM 1
 # else
 #  define R_RANDOM 1
